package com.gic.jpa.base;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.PostConstruct;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import java.util.ArrayList;
import java.util.List;

/**
 * @author 蘑小菇
 * @Description:
 * @date 2018/11/28 16:09
 */
public abstract class BaseServiceImpl<T> {

    private final static Logger logger = LoggerFactory.getLogger(BaseServiceImpl.class);

    private static final int SIZE = 100;

    @Autowired
    private EntityManagerFactory entityManagerFactory;

    private EntityManager em;
    private EntityTransaction transaction ;

    @PostConstruct
    public void  postConstruct(){
        logger.info("I'm  init  method  using  @PostConstrut...." );
        em = entityManagerFactory.createEntityManager();
        transaction = em.getTransaction();
        logger.info("createEntityManager ...." );
    }


    public List<T> batchCreate(List<T> list) {
        List<List<T>> batchList = averageAssign(list, SIZE);
        for (List<T> ts : batchList) {
            transaction.begin();
            create(ts);
            transaction.commit();
            em.clear();
        }

        return list;
    }

    public List<T> batchUpdate(List<T> list) {
        List<List<T>> batchList = averageAssign(list, SIZE);
        for (List<T> ts : batchList) {
            transaction.begin();
            update(ts);
            transaction.commit();
            em.clear();
        }
        return list;
    }

    private void update(List<T> singleList) {
        for (T t : singleList) {
            this.em.merge(t);
        }
        this.em.flush();
    }

    private void create(List<T> singleList) {
        for (T t : singleList) {
            this.em.persist(t);
        }
        this.em.flush();
    }


    /**
     * 将一个list均分成n个list,主要通过偏移量来实现的
     * @param source
     * @return
     */
    public static <T> List<List<T>> averageAssign(List<T> source, int n) {
        List<List<T>> result = new ArrayList<>();
        //先计算出余数
        int remaider = source.size() % n;
        //然后是商
        int number = source.size() / n;
        //偏移量
        int offset = 0;
        for (int i = 0; i < n; i++) {
            List<T> value = null;
            if (remaider > 0) {
                value = source.subList(i * number + offset, (i + 1) * number + offset + 1);
                remaider--;
                offset++;
            } else {
                value = source.subList(i * number + offset, (i + 1) * number + offset);
            }
            result.add(value);
        }
        return result;
    }

}
