package com.gic.jpa.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.jpa.base.BaseServiceImpl;
import com.gic.jpa.entity.Order;
import com.gic.jpa.entity.OrderItem;
import com.gic.jpa.entity.OrderLogistics;
import com.gic.jpa.entity.OrderRefundLogistics;
import com.gic.jpa.repository.OrderItemRepository;
import com.gic.jpa.repository.OrderLogisticsRepository;
import com.gic.jpa.repository.OrderRefundLogisticsRepository;
import com.gic.jpa.repository.OrderRepository;
import com.gic.jpa.service.OrderService;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.concurrent.CopyOnWriteArrayList;


/**
 * @author 蘑小菇
 * @Description:
 * @date 2018/11/20 11:22
 */

@Component
public class OrderServiceImpl extends BaseServiceImpl implements OrderService {

    private static Logger logger = LoggerFactory.getLogger(OrderServiceImpl.class);

    private static final int limit = 400;

    /**
     * 折扣默认数据
     */
    private static final String DEFUALT_DISCOUNT = "{\"503\":0.00,\"504\":0.00,\"400\":0.00,\"501\":0.00,\"502\":0.00}";

    /**
     * 支付默认
     */
    private static final String DEFUALT_PAYINFO = "{\"202\":0.00,\"300\":0.00,\"201\":0.00,\"100\":0.00}";


    @Autowired
    private OrderRepository orderRepository;
    @Autowired
    private OrderItemRepository orderItemRepository;
    @Autowired
    private OrderLogisticsRepository orderLogisticsRepository;
    @Autowired
    private OrderRefundLogisticsRepository orderRefundLogisticsRepository;

    @Override
    public void formatOrder()   {

        long count = orderRepository.count();
        long total = (count / limit) + 1;

        for (int i = 0; i <= total; i++) {

            Pageable pageable = new PageRequest(i, limit);
            Page<Order> page = orderRepository.findAll(pageable);

            if (page == null || CollectionUtils.isEmpty(page.getContent())) {
                continue;
            }

            CopyOnWriteArrayList<Order> copyList = new CopyOnWriteArrayList(page.getContent());

            for (Order order : copyList) {

                if (isJSON(order.getAmountPayableInfo()) && isJSON(order.getDiscountAmountInfo())) {
                    copyList.remove(order);
                }
                order.setAmountPayableInfo(formatPayinfo(order.getAmountPayableInfo()));
                order.setDiscountAmountInfo(formatDiscount(order.getDiscountAmountInfo()));
                formatAddress(order);
                order.setOrderVersion("version_1.0");
            }

            if (!CollectionUtils.isEmpty(copyList)) {
                orderRepository.save(copyList);
            }

        }

    }

    private void formatAddress(Order order) {
        if (StringUtils.isNotBlank(order.getAddressInfo())) {
            String fullName = order.getProvinceName() + order.getCityName() + order.getAreaName();
            String addr =order.getAddressInfo().replace(fullName,"");
            order.setAddressInfo(addr);
        }

    }

    @Override
    public void formatOrderItem() {
        long count = orderItemRepository.count();
        long total = (count / limit) + 1;

        for (int i = 0; i <= total; i++) {
            Pageable pageable = new PageRequest(i, limit);
            Page<OrderItem> page = orderItemRepository.findAll(pageable);
            if (page == null || CollectionUtils.isEmpty(page.getContent())) {
                continue;
            }

            CopyOnWriteArrayList<OrderItem> copyList = new CopyOnWriteArrayList(page.getContent());

            for (OrderItem order : copyList) {

                if (isJSON(order.getAmountPayableInfo()) && isJSON(order.getDiscountAmountInfo())) {
                    copyList.remove(order);
                }
                order.setAmountPayableInfo(formatPayinfo(order.getAmountPayableInfo()));
                order.setDiscountAmountInfo(formatDiscount(order.getDiscountAmountInfo()));

            }

            if (!CollectionUtils.isEmpty(copyList)) {
                orderItemRepository.save(copyList);
            }


        }

    }


    /**
     * // 优惠折扣详情 400会员等级折扣 500活动促销(501折扣活动 502满立减活动 503包邮活动 504阶梯价活动)
     * @param string
     * @return
     */
    private String formatDiscount(String string) {
        JSONObject json = JSONObject.parseObject(DEFUALT_DISCOUNT);
        if (StringUtils.isBlank(string)) {
            return DEFUALT_DISCOUNT;
        }

        if (isJSON(string)) {
            return string;
        }


        formateJSON(string, json);

        return json.toJSONString();
    }

    private void formateJSON(String string, JSONObject json) {
        String[] array = string.split("\\|\\|");
        for (String s : array) {
            if (StringUtils.isBlank(s)) {
                continue;
            }
            String[] item = s.split("=");
            if (ArrayUtils.isEmpty(item)) {
                continue;
            }

            if (StringUtils.isNotBlank(item[0]) && item.length == 2) {
                json.remove(item[0]);
                json.put(item[0], new BigDecimal(item[1]).setScale(2,BigDecimal.ROUND_HALF_DOWN));
            }
        }
    }

    private String formatPayinfo(String pay) {

        JSONObject json = JSONObject.parseObject(DEFUALT_PAYINFO);
        if (StringUtils.isBlank(pay)) {
            return json.toJSONString();
        }

        if (isJSON(pay)) {
            return pay;
        }

        formateJSON(pay, json);

        return json.toJSONString();
    }


    private boolean isJSON(String str) {
        try {
            JSONObject.parse(str);
            return true;
        } catch (Exception e) {
            return false;
        }
    }


    @Override
    public void formatLogistics() {

        handlerOrderLogistics();
        handlerOrderRefundLogistics();
    }

    private void handlerOrderLogistics(){
        long count = orderLogisticsRepository.count();
        long total = (count / limit) + 1;

        for (int i = 0; i <= total; i++) {
            Pageable pageable = new PageRequest(i, limit);
            Page<OrderLogistics> page = orderLogisticsRepository.findAll(pageable);
            if (page == null || CollectionUtils.isEmpty(page.getContent())) {
                continue;
            }

            CopyOnWriteArrayList<OrderLogistics> copyList = new CopyOnWriteArrayList(page.getContent());

            for (OrderLogistics order : copyList) {
                if (isJSON(order.getLogisticsInformation()) ) {
                    copyList.remove(order);
                }
                order.setLogisticsInformation(formateTraces(order.getLogisticsInformation()));
            }

            if (CollectionUtils.isEmpty(copyList)) {
                continue;
            }
            orderLogisticsRepository.save(copyList);
        }
    }

    private void handlerOrderRefundLogistics(){
        long count = orderRefundLogisticsRepository.count();
        long total = (count / limit) + 1;

        for (int i = 0; i <= total; i++) {
            Pageable pageable = new PageRequest(i, limit);
            Page<OrderRefundLogistics> page = orderRefundLogisticsRepository.findAll(pageable);
            if (page == null || CollectionUtils.isEmpty(page.getContent())) {
                continue;
            }

            CopyOnWriteArrayList<OrderRefundLogistics> copyList = new CopyOnWriteArrayList(page.getContent());

            for (OrderRefundLogistics order : copyList) {
                if (isJSON(order.getLogisticsInformation()) ) {
                    copyList.remove(order);
                }
                order.setLogisticsInformation(formateTraces(order.getLogisticsInformation()));
            }

            if (!CollectionUtils.isEmpty(copyList)) {
                orderRefundLogisticsRepository.save(copyList);
            }
        }
    }


    private static final String empty= "[]";

    private String formateTraces(String info){


        JSONObject trace = new JSONObject(true);
        trace.put("EBusinessID","1283242");
        trace.put("LogisticCode","courierNumber");
        trace.put("ShipperCode","logisticsCompanyCode");
        trace.put("State",0);
        trace.put("Success",true);
        if (StringUtils.isBlank(info) || empty.equals(info)){
            trace.put("Reason","暂无轨迹信息");
        }
        try {
            trace.put("Traces", JSON.parseArray(info));
        }catch (Exception e){
            logger.info("source:{}",info);
            trace.put("Traces", new JSONArray());
        }
        return trace.toJSONString();
    }
}
